/* Ce client UDP : 
  1/ prend en parametre une adresse IP (exemple d'utilisation : ./client_udp 127.0.0.1)
  2/ envoie un paquet UDP avec le mot "request" a un serveur ecoutant sur le port 3000 et sur l'adresse indiquee en parametre
  3/ lit la reponse , affiche celle ci , avec le port et l'adresse de l'emetteur de la reponse 
*/

#include <stdio.h>
#include <string.h>
#include <strings.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <sys/time.h>
#include <termios.h>

/* definition des constantes */
#define MAXLINE 1024

/**
 * Permet l'activation du mode raw sur une console Linux
 *
 */
void mode_raw(int activer)
{


    static struct termios cooked;
    static int raw_actif = 0;

    if (raw_actif == activer)
        return;

    if (activer)
    {
        struct termios raw;

        tcgetattr(STDIN_FILENO, &cooked);

        raw = cooked;
        cfmakeraw(&raw);
        tcsetattr(STDIN_FILENO, TCSANOW, &raw);
    }
    else
        tcsetattr(STDIN_FILENO, TCSANOW, &cooked);

    raw_actif = activer;

}

/**
 * Lit de facon non bloquant une touche du clavier
 * Retourne -1 si il n'y a pas de touche dans le buffer
 * sinon retourne la valeur de la touche
 */
int readKeyBoard()
{
          struct timeval tv = { 0, 0 };
          fd_set readfds;

          FD_ZERO(&readfds);
          FD_SET(STDIN_FILENO, &readfds);

          if (select(STDIN_FILENO + 1, &readfds, NULL, NULL, &tv) == 1)
          {
                  return getchar();
          }
          else
          {
                  return -1;
          }
}

int main(int argc, char *argv[])
{
	int sockServer;
	int  nread, nsend;
	struct sockaddr_in addrServer, addrClient;
	char buffer[MAXLINE+1];
	int ports[15];
	int i;
	int touche = 0;

	for (i=0; i<15; i++){
		ports[i]=0;
	}

	/* SOCKET creation de socket */
	if ( (sockServer = socket(AF_INET, SOCK_DGRAM, 0)) < 0) 
	{
		perror("Socket creation error");
		return -1;
	}
	
	/* SEND initialiser l’adresse du serveur  */
	/* type d’adresse  INET */
	addrServer.sin_family = AF_INET;
	addrServer.sin_port = htons(3000);
	
	/* contruction d’adresse IP binaire avec inet_pton */
	/* l’adresse est donnee en argument « x.x.x.x » */
	if((inet_pton(AF_INET,argv[1],&addrServer.sin_addr)) <= 0) {
		perror("Address creation error");
		return -1;
	}
	bzero(addrServer.sin_zero, 8);

	bind(sockServer, (struct sockaddr *) &addrServer, sizeof(addrServer));
	
	/* RECEIVE lecture de la reponse */
	struct sockaddr_in from;
	unsigned int fromlen = sizeof(from);
     
	do {	
		nread = recvfrom(sockServer, buffer, MAXLINE, 0, (struct sockaddr *) &from,&fromlen);
	    	if (nread < 0) {
			perror("Read error");
			return -1;
	   	}
		buffer[nread]='\0';
		for (i=0; i<15; i++){
			if (ports[i]==0){
				ports[i]=ntohs(from.sin_port);
				break;
			}
		}
	}
	while (strcmp(buffer, "last") != 0);
	mode_raw(1);
	for(i=0; i<15; i++){
		if (ports[i] != 0){		
			// lecture non bloquante d'une touche
			touche = readKeyBoard();

			// Interpretation des touches

			// La touche q est la touche de sortie du programme
			if (touche == 'q' || touche == 'Q')
			{
				for(i=0; i<15; i++){
					if (ports[i] != 0){					
						/* SEND initialiser l’adresse du serveur  */
						/* type d’adresse  INET */
						addrClient.sin_family = AF_INET;
						addrClient.sin_port = htons(ports[i]);
						/* contruction d’adresse IP binaire avec inet_pton */
						/* l’adresse est donnee en argument « x.x.x.x » */
						if((inet_pton(AF_INET,argv[1],&addrClient.sin_addr)) <= 0) {
							perror("Address creation error");
							return -1;
						}
						bzero(addrClient.sin_zero, 8);			
						nsend = sendto(sockServer, "die", 3, 0, (struct sockaddr *)&addrClient, sizeof(addrClient));
						if (nsend < 0) {
							perror("Request error");
							return -1;
						}
					}
				}
				break;
			}		
			sleep(1);

			/* SEND initialiser l’adresse du serveur  */
			/* type d’adresse  INET */
			addrClient.sin_family = AF_INET;
			addrClient.sin_port = htons(ports[i]);
			/* contruction d’adresse IP binaire avec inet_pton */
			/* l’adresse est donnee en argument « x.x.x.x » */
			if((inet_pton(AF_INET,argv[1],&addrClient.sin_addr)) <= 0) {
				perror("Address creation error");
				return -1;
			}
			bzero(addrClient.sin_zero, 8);
			/* envoi d’un packet avec le mot request a l'interieur */
			nsend = sendto(sockServer, "toYou", 5, 0, (struct sockaddr *)&addrClient, sizeof(addrClient));
			if (nsend < 0) {
				perror("Request error");
				return -1;
			}
		}

		if (i==14)
			i=-1;
	}
	mode_raw(0);
     
	return 0;
     
}
