/**
 * Type énuméré pour les erreurs contextuelles.
 * Ce type énuméré définit toutes les erreurs contextuelles possibles et 
 * permet l'affichage des messages d'erreurs pour la passe 2.
 */

// -------------------------------------------------------------------------
// A COMPLETER, avec les différents types d'erreur et les messages d'erreurs 
// correspondants
// -------------------------------------------------------------------------

package ProjetCompil.Verif.Src;

public enum ErreurContext {
	/**
	 * Identifie les erreurs venant d'une variable non declarée
	 */
	ErreurVariableNonDeclaree,
	/**
	 * Identifie les erreurs venant de l'utilisation d'un indice de tableau qui n'est pas de type entier
	 */
	ErreurBorneNonEntier,
	/**
	 * Identifie les erreurs venant de la définition d'un intervalle négatif
	 */
	ErreurBorneDefinitionIntervalleInverse,
	/**
	 * Identifie les erreurs venant de la définition d'un intervalle nul
	 */
	ErreurBorneDefinitionIntervalleVide,
	/**
	 * Identifie les erreurs venant de l'utilisation d'un type non définit
	 */
	ErreurTypeNonDeclare,
	/**
	 * Identifie les erreurs venant d'une redéfinition d'une variable
	 */
	ErreurRedefinitionVariable,
	/**
	 * Identifie les erreurs venant de l'affectation d'une valeur dans un identificateur qui n'est pas une variable
	 */
	ErreurIdentificateurVarAttendu,
	/**
	 * Identifie les erreurs venant de l'utilisateur d'une affectation incompatible
	 */
	ErreurAffectionIncompatible,
	/**
	 * Identifie les erreurs venant de l'utilisation d'un operateur avec des types incompatibles
	 */
	ErreurOperationIncompatible,
	/**
	 * Identifie les erreurs venant de l'utilisation d'un identifiant qui n'est ni une constante ni une variable
	 */
	ErreurIdentificateurVarOuConstanteAttendu,
	/**
	 * Identifie les erreurs venant d'une mauvaise définition du pas pour la boucle for
	 */
	ErreurPasFor,
	/**
	 * Identifie les erreurs venant de l'utilisation d'une condition sur un element qui n'est pas de type boolean
	 */
	ErreurCondition,
	/**
	 * Identifie les erreurs venant de la mauvaise utilisation de la "fonction" Write
	 */
	ErreurWrite,
	/**
	 * Identifie les erreurs venant de la mauvaise utilisation de la "fonction" Lecture
	 */
	ErreurLecture,
	/**
	 * Identifie les erreurs qui sont en attente d'être définie
	 * Utilisé que lors de la phase de développement
	 */
   ErreurNonRepertoriee;

	/**
	 * 
	 * @param s Chaine de caractère qui comporte des informations sur l'erreur
	 * @param numLigne Numéro de la ligne du programme qui a levé l'erreur
	 * @throws ErreurVerif Pour permettre de notifié qu'une erreur de syntaxe est survenue
	 */
   void leverErreurContext(String s, int numLigne) throws ErreurVerif {
      System.err.println("Erreur contextuelle : ");
      switch (this) {
      	case ErreurVariableNonDeclaree:
      		System.err.print("La variable " + s + " n'a pas été préalablement définie");
      	break;
      	case ErreurBorneNonEntier:
      		System.err.print("La borne utilisée (\"" + s + "\") n'est pas de type entier");
      	break;
      	case ErreurBorneDefinitionIntervalleInverse:
      		System.err.print("La borne inférieur est plus grande que la borne supérieur");
      	break;
      	case ErreurBorneDefinitionIntervalleVide:
      		System.err.print("La borne inférieur et la borne supérieur sont égales");
      	break;
      	case ErreurTypeNonDeclare:
      		System.err.print("Le type " + s + " n'existe pas");
      	break;
      	case ErreurRedefinitionVariable:
      		System.err.print("La variable " + s + " a déjà été définie");
      	break;
      	case ErreurIdentificateurVarAttendu:
      		System.err.print("Un nom de variable est attendus, \"" + s + "\" n'en n'est pas");
      	break;
      	case ErreurAffectionIncompatible:
      		System.err.print("La variable n'est pas compatible avec l'expression  : " + s);
      	break;
      	case ErreurOperationIncompatible:
      		System.err.print("L'operateur \""+ s +"\" n'est pas compatible avec les types de variable utilisés");
      	break;
      	case ErreurIdentificateurVarOuConstanteAttendu:
      		System.err.print("Un nom de variable ou une constante est attendus, \"" + s + "\" n'en n'est pas");
      	break;
      	case ErreurPasFor:
      		System.err.print("Le pas de la boucle for doit être un interval");
      	break;
      	case ErreurCondition:
      		System.err.print("La condition de l'instruction \"" + s + "\" doit être booléenne");
      	break;
      	case ErreurWrite:
      		System.err.print("L'instruction Write attends des expresssions de type Interval, Réel ou Chaine de caractère. Expression de type \""+s+"\" utilisée");
      	break;
      	case ErreurLecture:
      		System.err.print("L'instruction Read attends des expresssions de type Interval ou Réel. Expression de type \""+s+"\" utilisée");
      	break;
      	
         default:
            System.err.print("non repertoriee");
      }
      System.err.println(" ... ligne " + numLigne);
      throw new ErreurVerif();
   }

}


