package ProjetCompil.Verif.Src;

import ProjetCompil.Global.Src.*;

/**
 * La classe ReglesTypage permet de définir les différentes règles 
 * de typage du langage JCas.
 */

public class ReglesTypage {

   /**
    * Teste si le type t1 et le type t2 sont compatibles pour l'affectation, 
    * c'est à dire si on peut affecter un objet de t2 à un objet de type t1.
    */

   static ResultatAffectCompatible affectCompatible(Type t1, Type t2) throws ErreurReglesTypage {
	   ResultatAffectCompatible result = new ResultatAffectCompatible();

		switch (t1.getNature()) {
			case Interval:
				if (t2.getNature() == NatureType.Interval)
					result.setOk(true);
				else
					result.setOk(false);
				break;

			case Boolean:
				if (t2.getNature() == NatureType.Boolean)
					result.setOk(true);
				else
					result.setOk(false);
				break;

			/*case Integer:
				if (t2.getNature() == NatureType.Integer)
					result.setOk(true);
				else
					result.setOk(false);*/
			
			case Real:
				if (t2.getNature() == NatureType.Real)
					result.setOk(true);
				else if (t2.getNature() == NatureType.Interval) {
					result.setConv2(true); // On convertis l'interval en Real
					result.setOk(true);
				}
				else 
					result.setOk(false);
				break;

			case Array:
				if (t2.getNature() == NatureType.Array
						&& t1.getIndice().getBorneInf() == t2.getIndice().getBorneInf()
						&& t1.getIndice().getBorneSup() == t2.getIndice().getBorneSup()) {
					result = affectCompatible(t1.getElement(), t2.getElement()); // Recursif car on peux faire des tableaux multi-dimensionnels
				}
				else
					result.setOk(false);
				break;

			default:
				throw new ErreurReglesTypage();
		}
		return result;
   }

   /**
    * Teste si le type t1 et le type t2 sont compatible pour l'opération 
    * binaire représentée dans noeud.
    */

   static ResultatBinaireCompatible binaireCompatible(Noeud noeud, Type t1, Type t2) throws ErreurReglesTypage {
	   ResultatBinaireCompatible result = new ResultatBinaireCompatible();

		switch (noeud) {
			case Et:
			case Ou:
				if (t1.getNature() == t2.getNature() && t1.getNature() == NatureType.Boolean){
					result.setOk(true);
					result.setTypeRes(Type.Boolean);
				}
				else
					result.setOk(false);
			break;
			case Egal:
			case Inf:
			case Sup:
			case InfEgal:
			case SupEgal:
			case NonEgal:
				switch(t1.getNature()){
					case Interval:
					case Real:
						if (t2.getNature() == NatureType.Interval || t2.getNature() == NatureType.Real){
							result.setOk(true);
							result.setTypeRes(Type.Boolean);
						}
						else
							result.setOk(false);
					break;
					
					default:
						result.setOk(false);
				}
			break;
			case Plus:
			case Moins:
			case Mult:
				switch (t1.getNature()) {
					case Interval:
						if (t2.getNature() == NatureType.Interval){
							result.setOk(true);
							result.setTypeRes(Type.Integer);
						}
						else if (t2.getNature() == NatureType.Real){
							result.setOk(true);
							result.setTypeRes(Type.Real);
						}
						else{
							result.setOk(false);
						}
					break;
					case Real:
						if (t2.getNature() == NatureType.Interval || t2.getNature() == NatureType.Real){
							result.setOk(true);
							result.setTypeRes(Type.Real);
						}
						else
							result.setOk(false);
					break;
					
					default:
						result.setOk(false);
				}
			break;
			case Quotient:
			case Reste:
				if (t1.getNature() == t2.getNature() && t1.getNature() == NatureType.Interval){
					result.setOk(true);
					result.setTypeRes(Type.Integer);
				}
				else
					result.setOk(false);
			break;
			case DivReel:
				switch(t1.getNature()){
					case Interval:
					case Real:
						if (t2.getNature() == NatureType.Interval || t2.getNature() == NatureType.Real){
							result.setOk(true);
							result.setTypeRes(Type.Real);
						}
						else{
							result.setOk(false);
						}
					break;
					
					default:
						result.setOk(false);
					break;
				}
			break;
			case Index:
				if (t1.getNature() == NatureType.Array	&& t2.getNature() == NatureType.Interval) {
					result.setOk(true);
					result.setTypeRes(t1.getElement());
				}
				else
					result.setOk(false);
			break;
			
			default:
				throw new ErreurReglesTypage();
		}
		
		return result;
   }

   /**
    * Teste si le type t est compatible pour l'opération binaire représentée 
    * dans noeud.
    */
   static ResultatUnaireCompatible unaireCompatible(Noeud noeud, Type t) throws ErreurReglesTypage {
	   ResultatUnaireCompatible result = new ResultatUnaireCompatible();
	   switch (noeud){
	   		case Non:
	   			if (t.getNature() == NatureType.Boolean){
	   				result.setOk(true);
					result.setTypeRes(Type.Boolean);
	   			}
	   			else
	   				result.setOk(false);
	   		break;
	   		case PlusUnaire:
	   		case MoinsUnaire:
	   			if (t.getNature() == NatureType.Interval){
	   				result.setOk(true);
					result.setTypeRes(Type.Integer);
	   			}
	   			else if (t.getNature() == NatureType.Real){
	   				result.setOk(true);
					result.setTypeRes(Type.Real);
	   			}
	   			else
	   				result.setOk(false);
	   			break;
      		default:
      			throw new ErreurReglesTypage();
      }
	   
	   return result;
      
   }
         
}

