package ProjetCompil.Verif.Src;

import ProjetCompil.Global.Src.*;
 
/**
 * Cette classe permet de réaliser la vérification et la décoration 
 * de l'arbre abstrait d'un programme.
 */
public class Verif {

   private Environ env; // L'environnement des identificateurs

   /**
    * Constructeur.
    */
   public Verif() {
      env = new Environ();
   }

   /**
    * Vérifie les contraintes contextuelles du programme correspondant à 
    * l'arbre abstrait a, qui est décoré et enrichi. 
    * Les contraintes contextuelles sont décrites 
    * dans Context.txt.
    * En cas d'erreur contextuelle, un message d'erreur est affiché et 
    * l'exception ErreurVerif est levée.
    */
   public void verifierDecorer(Arbre a) throws ErreurVerif {
      verifier_PROGRAMME(a);
   }

   /**
    * Initialisation de l'environnement avec les identificateurs prédéfinis.
    */
   private void initialiserEnv() {
      Defn def;
      // integer
      def = Defn.creationType(Type.Integer);
      def.setGenre(Genre.PredefInteger);
      env.enrichir("integer", def);
      
      // ------------
      // A COMPLETER
      // ------------
      def = Defn.creationType(Type.Boolean);
      def.setGenre(Genre.PredefBoolean);
      env.enrichir("boolean", def);
      
      def = Defn.creationConstBoolean(false);
      def.setGenre(Genre.PredefFalse);
      env.enrichir("false", def);
      
      def = Defn.creationConstBoolean(true);
      def.setGenre(Genre.PredefTrue);
      env.enrichir("true", def);
      
      def = Defn.creationConstInteger(java.lang.Integer.MAX_VALUE);
      def.setGenre(Genre.PredefMaxInt);
      env.enrichir("max_int", def);
      
      def = Defn.creationType(Type.Real);
      def.setGenre(Genre.PredefReal);
      env.enrichir("real", def);
   }

   //NOTE: Il existe des constantes ?
   
   private boolean estFeuille(Arbre a){
	   return (a.getArite() == 0);
   }
   
   
   /**************************************************************************
    * PROGRAMME
    **************************************************************************/
   private void verifier_PROGRAMME(Arbre a) throws ErreurVerif {
      initialiserEnv();
      verifier_LISTE_DECL(a.getFils1());
      verifier_LISTE_INST(a.getFils2());
   }
   
   /*
    * PARTIE DECLARATION DE VARIABLES
    */
   
   private int verifier_CONST_ENT(Arbre a) throws ErreurVerif {
	   if (!estFeuille(a))
			throw new ErreurInterneVerif("Abre incorrect dans verifier_ENTIER");

		a.setDecor(new Decor(Type.Integer));
		return a.getEntier();
   }
   
   private int verifier_IDENT_CONST(Arbre a) throws ErreurVerif {
	   if (!estFeuille(a))
			throw new ErreurInterneVerif("Abre incorrect dans verifier_IDENT_CONST");
	
		Defn def = env.chercher(a.getChaine()); // On vérifie si c'est bien un type définit
		if (def == null) { // Si la variable n'a pas été définie
			ErreurContext.ErreurVariableNonDeclaree.leverErreurContext(a.getChaine(), a.getNumLigne());
		} //NOTE: Pas sur que ça soit utilisé uniquement pour les intervals
		else if ((def.getType() != Type.Integer) || (def.getNature() != NatureDefn.ConstInteger))  // Si ce n'est pas un entier
			ErreurContext.ErreurBorneNonEntier.leverErreurContext(a.getChaine(), a.getNumLigne());
		a.setDecor(new Decor(def));
		return def.getValeurInteger();
   }
   
   private int verifier_CONST(Arbre a) throws ErreurVerif {
		switch (a.getNoeud()) {
			case Ident:
				return verifier_IDENT_CONST(a);
			case Entier:
				return verifier_CONST_ENT(a);
				
			default:
				throw new ErreurInterneVerif("Abre incorrect dans verifier_CONST");
		}
   }
   
   private int verifier_CONSTANTE(Arbre a) throws ErreurVerif {
		switch (a.getNoeud()) {
			case PlusUnaire:
				return new Integer(+verifier_CONST(a.getFils1()));
			case MoinsUnaire:
				return new Integer(-verifier_CONST(a.getFils1()));
				
			default:
				return verifier_CONST(a);
		}
	}

   private Type verifier_TYPE_INTERVALLE(Arbre a) throws ErreurVerif {
	   int borneInf = verifier_CONSTANTE(a.getFils1());
	   int borneSup = verifier_CONSTANTE(a.getFils2());

	   if (borneInf > borneSup)
		   ErreurContext.ErreurBorneDefinitionIntervalleInverse.leverErreurContext("", a.getNumLigne());
	   else if (borneSup == borneInf)
		   ErreurContext.ErreurBorneDefinitionIntervalleVide.leverErreurContext("", a.getNumLigne());
	   return Type.creationInterval(borneInf, borneSup);
   }
   
   private Type verifier_IDENT_TYPE(Arbre a) throws ErreurVerif {
	   if (!estFeuille(a))
			throw new ErreurInterneVerif("Abre incorrect dans verifier_IDENT_TYPE");
		Defn def = env.chercher(a.getChaine()); // On vérifie si c'est bien un type définit

		// Regarde dans la table d'environnement si le type a bien été défini et si c'est bien un type (pas une valeur comme true)
		if (def.getNature() != NatureDefn.Type || def == null) 
			ErreurContext.ErreurTypeNonDeclare.leverErreurContext(a.getChaine(), a.getNumLigne());
		a.setDecor(new Decor(def)); // On décore l'abre
		
		return def.getType();
   }
   
   private Type verifier_TYPE(Arbre a) throws ErreurVerif {
	   switch (a.getNoeud()) {
  		case Ident: // Type
  			return verifier_IDENT_TYPE(a);
  		case Intervalle:
  			return verifier_TYPE_INTERVALLE(a);
  		case Tableau:
  			return Type.creationArray(verifier_TYPE_INTERVALLE(a.getFils1()), verifier_TYPE(a.getFils2()));
  		default:
  			throw new ErreurInterneVerif("Abre incorrect dans verifier_TYPE, type inconnu");
	   }
   }
   
   /**
    * Permet de vérifier un identificateur comportant un nom de variable
    * @param a
    * @param type
    * @throws ErreurVerif
    */
   private void verifier_IDENT_DEF_VAR(Arbre a, Type type) throws ErreurVerif {
	   if (!estFeuille(a))
			throw new ErreurInterneVerif("Abre incorrect dans verifier_IDENT_DEF_VAR");
		if (env.chercher(a.getChaine()) != null)  // Si la variable a déjà été définie
			ErreurContext.ErreurRedefinitionVariable.leverErreurContext(a.getChaine(), a.getNumLigne());
		Defn def = Defn.creationVar(type); // On ajoute la définition de la variable
		a.setDecor(new Decor(def));
		env.enrichir(a.getChaine(), def);
   }
  
   private void verifier_LIST_IDENT(Arbre a, Type type) throws ErreurVerif {
	   if (a.getFils1().getNoeud() == Noeud.ListeIdent)
		   verifier_LIST_IDENT(a.getFils1(), type); // On vérifié les déclarations suivantes
	   verifier_IDENT_DEF_VAR(a.getFils2(), type); // N'est pas spécialement un type
   }
   
   private void verifier_DECL(Arbre a) throws ErreurVerif {
	   Type type = verifier_TYPE(a.getFils2());
	   verifier_LIST_IDENT(a.getFils1(), type);
   }
   
   /**************************************************************************
    * LISTE_DECL
    **************************************************************************/
   private void verifier_LISTE_DECL(Arbre a) throws ErreurVerif {
	   if (a.getNoeud() != Noeud.Vide) {
		   if (a.getFils1().getNoeud() == Noeud.ListeDecl)
			   verifier_LISTE_DECL(a.getFils1()); // On vérifié les déclarations suivantes
		   verifier_DECL(a.getFils2());
		}
   }
   
   /*
    * PARTIE INSTRUCTIONS
    */
   

   /**************************************************************************
    * LISTE_INST
    **************************************************************************/
	void verifier_LISTE_INST(Arbre a) throws ErreurVerif {
		switch (a.getNoeud()) {
			case Vide: 
			break; 
			case ListeInst:
				verifier_LISTE_INST(a.getFils1()); 
				verifier_INST(a.getFils2());
			break;
			
			default: 
				throw new ErreurInterneVerif("Arbre incorrect dans verifier_LISTE_INST"); 
		} 
	}
	
	private void verifier_IDENT_PLACE(Arbre a) throws ErreurVerif {
		if (!estFeuille(a))
			throw new ErreurInterneVerif("Abre incorrect dans verifier_IDENT_PLACE");
		Defn def = env.chercher(a.getChaine()); // On vérifie si c'est bien un type définit

		// Regarde dans la table d'environnement si le type a bien été défini et si c'est bien un type (pas une valeur comme true)
		if (def == null) 
			ErreurContext.ErreurTypeNonDeclare.leverErreurContext(a.getChaine(), a.getNumLigne());
		else if (def.getNature() != NatureDefn.Var) {
			ErreurContext.ErreurIdentificateurVarAttendu.leverErreurContext(a.getChaine(), a.getNumLigne());
		}
		
		a.setDecor(new Decor(def, def.getType()));// On décore l'abre
			
	  }
	
	private void verifier_PLACE(Arbre a) throws ErreurVerif {
		Type t1, t2;
		
		switch (a.getNoeud()) {
			case Ident:
				return verifier_IDENT_PLACE(a);
			break; 
			case Index:
				t1 = verifier_PLACE(a.getFils1());
				t2 = verifier_EXP(a.getFils2()); //TODO Coder cette partie
				//TODO Coder la comparaison des types
			break;
			
			default: 
				throw new ErreurInterneVerif("Arbre incorrect dans verifier_PLACE"); 
		}
	}
	
	void verifier_INST(Arbre a) throws ErreurVerif {
		Type t1, t2;
		switch (a.getNoeud()) {
			case Nop: 
			break; 
			case Affect:
				t1 = verifier_PLACE(a.getFils1());
				t2 = verifier_EXP(a.getFils2());
				ResultatAffectCompatible comp = ReglesTypage.affectCompatible(t1, t2);
				if (!comp.getOk())
					ErreurContext.ErreurAffectionIncompatible.leverErreurContext("", a.getNumLigne());
				if (comp.getConv2()){
					Arbre nc = Arbre.creation1(Noeud.Conversion, a.getFils(2), a.getNumLigne());
					a.setFils(2, nc);
					nc.setDecor(new Decor(t1));
				}
					
				a.setDecor(new Decor(t1));
			break;
			case Pour:
				verifier_POUR(a);
			break;
			case TantQue:
				verifier_TANT_QUE(a);
			break;
			case Si:
				verifier_SI(a);
			break;
			case Ecriture:
				verifier_ECRITURE(a);
			break;
			case Lecture:
				verifier_LECTURE(a);
			break;
			case Ligne:
				verifier_LIGNE(a);
			break;
			
			default:
				throw new ErreurInterneVerif("Arbre incorrect dans verifier_INST");
		}
	}

}
