package ProjetCompil.Verif.Src;

import ProjetCompil.Global.Src.*;
 
/**
 * Cette classe permet de réaliser la vérification et la décoration 
 * de l'arbre abstrait d'un programme.
 */
public class Verif {

   private Environ env; // L'environnement des identificateurs

   /**
    * Constructeur.
    */
   public Verif() {
      env = new Environ();
   }

   /**
    * Vérifie les contraintes contextuelles du programme correspondant à 
    * l'arbre abstrait a, qui est décoré et enrichi. 
    * Les contraintes contextuelles sont décrites 
    * dans Context.txt.
    * En cas d'erreur contextuelle, un message d'erreur est affiché et 
    * l'exception ErreurVerif est levée.
    */
   public void verifierDecorer(Arbre a) throws ErreurVerif {
      verifier_PROGRAMME(a);
   }

   /**
    * Initialisation de l'environnement avec les identificateurs prédéfinis.
    */
   private void initialiserEnv() {
      Defn def;
      // integer
      def = Defn.creationType(Type.Integer);
      def.setGenre(Genre.PredefInteger);
      env.enrichir("integer", def);
      
      // ------------
      // A COMPLETER
      // ------------
      def = Defn.creationType(Type.Boolean);
      def.setGenre(Genre.PredefBoolean);
      env.enrichir("boolean", def);
      
      def = Defn.creationConstBoolean(false);
      def.setGenre(Genre.PredefFalse);
      env.enrichir("false", def);
      
      def = Defn.creationConstBoolean(true);
      def.setGenre(Genre.PredefTrue);
      env.enrichir("true", def);
      
      def = Defn.creationConstInteger(java.lang.Integer.MAX_VALUE);
      def.setGenre(Genre.PredefMaxInt);
      env.enrichir("max_int", def);
      
      def = Defn.creationType(Type.Real);
      def.setGenre(Genre.PredefReal);
      env.enrichir("real", def);
   }

   /**************************************************************************
    * PROGRAMME
    **************************************************************************/
   private void verifier_PROGRAMME(Arbre a) throws ErreurVerif {
      initialiserEnv();
      verifier_LISTE_DECL(a.getFils1());
      verifier_LISTE_INST(a.getFils2());
   }

   /**************************************************************************
    * TYPE
    **************************************************************************/
 /*  private void verifier_TYPE(Arbre a) throws ErreurVerif {
	   switch (a.getNoeud()) {
	   		case Ident:
	   			return verifier_IDENT_DECL_TYPE(a);
	   		case Intervalle:
	   			return verifier_TYPE_INTERVALLE(a);
	   		case Tableau:
	   			return Type.creationArray(verifier_TYPE_INTERVALLE(a.getFils1()),
					verifier_TYPE(a.getFils2()));
	   		default:
	   			throw new ErreurVerif("Abre incorrect détecté par verifier_TYPE");
	   }
   }*/
   
   private String verifier_IDENT(Arbre a, String type="") throws ErreurVerif {
	   if (a.getNoeud() != Noeud.Ident){ // Si on a pas la noeud de type IDENT
		   throw new ErreurInterneVerif("Noeud de type Ident attendus");
	   }
	   
	   
	   if (type == ""){
		   switch (a.getChaine()){
		   		case "integer":
		   		case "real":
		   		case "boolean":
		   			break;
		   		default:
		   			throw new ErreurInterneVerif("Type attendus");
		   			// TODO Gérer les tableaux
		   			break;
		   }
		   return a.getChaine();
	   }
	   else{
		   //a.set
		   
		   
		   return type;
	   }
   }
   
   private void verifier_LIST_IDENT(Arbre a, String type) throws ErreurVerif {
	   if (a.getFils1().getNoeud() == Noeud.ListeIdent)
		   verifier_LIST_IDENT(a.getFils1(), type); // On vérifié les déclarations suivantes
	   verifier_IDENT(a.getFils2(), type); // N'est spécialement un type
   }
   
   private void verifier_DECL(Arbre a) throws ErreurVerif {
	   String type = verifier_IDENT(a.getFils2()); // Devras être un type
	   verifier_LIST_IDENT(a.getFils1());
   }
   
   /**************************************************************************
    * LISTE_DECL
    **************************************************************************/
   private void verifier_LISTE_DECL(Arbre a) throws ErreurVerif {
	   if (a.getNoeud() != Noeud.Vide) {
		   if (a.getFils1().getNoeud() == Noeud.ListeDecl)
			   verifier_LISTE_DECL(a.getFils1()); // On vérifié les déclarations suivantes
		   verifier_DECL(a.getFils2());
		}
   }

   /**************************************************************************
    * LISTE_INST
    **************************************************************************/
   private void verifier_LISTE_INST(Arbre a) throws ErreurVerif {
      // A COMPLETER
   }

   // ------------------------------------------------------------------------
   // COMPLETER les operations de vérifications et de décoration pour toutes 
   // les constructions d'arbres
   // ------------------------------------------------------------------------

}
